from enum import Enum


class ActionType(str, Enum):
    ACTION_KIWI_SCOUT = "action_kiwi_scout"
    ACTION_LLM = "action_llm"
    ACTION_TEXT2TEXT_LINKEDIN = "action_text2text_linkedin"
    ACTION_TEXT2TEXT_PDF_PARSER = "action_text2text_pdf_parser"
    API = "api"
    AUDIO2TEXT_TRANSCRIPTION_ASSEMBLYAI = "audio2text_transcription_assemblyai"
    AUDIO2TEXT_TRANSCRIPTION_CARTESIA = "audio2text_transcription_cartesia"
    AUDIO2TEXT_TRANSCRIPTION_DEEPGRAM = "audio2text_transcription_deepgram"
    AUDIO2TEXT_TRANSCRIPTION_OPENAI = "audio2text_transcription_openai"
    AUDIO2TEXT_TRANSLATION_OPENAI = "audio2text_translation_openai"
    CLAUDE = "claude"
    CSV_MERGE = "csv_merge"
    DATABASE = "database"
    DEEPSEEK = "deepseek"
    EXTRACT_URLS = "extract_urls"
    FILE_UPLOADER_GEMINI = "file_uploader_gemini"
    FILE_UPLOADER_OPENAI = "file_uploader_openai"
    GEMINI = "gemini"
    GROK = "grok"
    IMG2IMG_BULK_EDIT_CLAID = "img2img_bulk_edit_claid"
    IMG2IMG_EDIT_OPENAI = "img2img_edit_openai"
    IMG2IMG_PHOTOSHOOT_CLAID = "img2img_photoshoot_claid"
    IMG2IMG_VARIATION_OPENAI = "img2img_variation_openai"
    JSON_FLATTEN = "json_flatten"
    LLM_OBJECTIVES_OPENAI = "llm_objectives_openai"
    MOCK_ACTION = "mock_action"
    MULTICLOUD_ROUTER = "multicloud_router"
    MULTICLOUD_ROUTER_SEARCH = "multicloud_router_search"
    MULTIMODAL_ASSISTANT_OPENAI = "multimodal_assistant_openai"
    MULTIMODAL_EXA_SEARCH = "multimodal_exa_search"
    MULTIMODAL_GOOGLE_SEARCH = "multimodal_google_search"
    MULTIMODAL_GRAPH_RAG_QUERY = "multimodal_graph_rag_query"
    MULTIMODAL_LLM_CHAT_WITH_VECTOR_SEARCH_OPENAI = "multimodal_llm_chat_with_vector_search_openai"
    MULTIMODAL_SANDBOX_QUERY = "multimodal_sandbox_query"
    MULTIMODAL_SERP_GOOGLE_SEARCH = "multimodal_serp_google_search"
    OPENAI = "openai"
    OPENAI_ASSISTANT = "openai_assistant"
    OPENAI_CHAT = "openai_chat"
    PLAYWRIGHT_LOGIN = "playwright_login"
    PLAYWRIGHT_SCRAPE = "playwright_scrape"
    TEXT2AUDIO_OPENAI = "text2audio_openai"
    TEXT2AUDIO_SPEECH_CARTESIA = "text2audio_speech_cartesia"
    TEXT2AUDIO_SPEECH_DEEPGRAM = "text2audio_speech_deepgram"
    TEXT2IMG_BUTTON_MIDJOURNEY_AI = "text2img_button_midjourney_ai"
    TEXT2IMG_DALLE_OPENAI = "text2img_dalle_openai"
    TEXT2IMG_FLUX = "text2img_flux"
    TEXT2IMG_GEMINI = "text2img_gemini"
    TEXT2IMG_GROK = "text2img_grok"
    TEXT2IMG_IMAGINE_MIDJOURNEY_AI = "text2img_imagine_midjourney_ai"
    TEXT2IMG_SEARCH_IMAGE = "text2img_search_image"
    TEXT2TEXT_API_CALL = "text2text_api_call"
    TEXT2TEXT_CONVERT_FILE_TO_PDF_OR_TXT = "text2text_convert_file_to_pdf_or_txt"
    TEXT2TEXT_IO_MAPPER = "text2text_io_mapper"
    TEXT2TEXT_LLM_CHAT_CLAUDE = "text2text_llm_chat_claude"
    TEXT2TEXT_LLM_CHAT_DEEPSEEK = "text2text_llm_chat_deepseek"
    TEXT2TEXT_LLM_CHAT_GEMINI = "text2text_llm_chat_gemini"
    TEXT2TEXT_LLM_CHAT_GROK = "text2text_llm_chat_grok"
    TEXT2TEXT_LLM_CHAT_GROK_COLLECTIONS = "text2text_llm_chat_grok_collections"
    TEXT2TEXT_LLM_CHAT_OPENAI = "text2text_llm_chat_openai"
    TEXT2TEXT_LLM_RESPONSES_OPENAI = "text2text_llm_responses_openai"
    TEXT2TEXT_READ_API_FILE = "text2text_read_api_file"
    TEXT2TEXT_READ_URL = "text2text_read_url"
    TEXT2TEXT_SEARCH_WEB = "text2text_search_web"
    TEXT2TEXT_SQL_RUNNER = "text2text_sql_runner"
    TEXT2TEXT_USER_INPUT = "text2text_user_input"
    TEXT2VIDEO_GEMINI_VEO_VIDEO_GENERATION = "text2video_gemini_veo_video_generation"
    TEXT2VIDEO_GROK = "text2video_grok"
    TEXT2VIDEO_SEARCH_VIDEO = "text2video_search_video"
    VIDEO2VIDEO_OPUS = "video2video_opus"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.runnable_type import RunnableType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ShareDataUpdate")


@_attrs_define
class ShareDataUpdate:
    """
    Attributes:
        data_id (None | str | Unset):
        runnable_type (None | RunnableType | Unset):
    """

    data_id: None | str | Unset = UNSET
    runnable_type: None | RunnableType | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        data_id: None | str | Unset
        if isinstance(self.data_id, Unset):
            data_id = UNSET
        else:
            data_id = self.data_id

        runnable_type: None | str | Unset
        if isinstance(self.runnable_type, Unset):
            runnable_type = UNSET
        elif isinstance(self.runnable_type, RunnableType):
            runnable_type = self.runnable_type.value
        else:
            runnable_type = self.runnable_type

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if data_id is not UNSET:
            field_dict["data_id"] = data_id
        if runnable_type is not UNSET:
            field_dict["runnable_type"] = runnable_type

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_data_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        data_id = _parse_data_id(d.pop("data_id", UNSET))

        def _parse_runnable_type(data: object) -> None | RunnableType | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                runnable_type_type_0 = RunnableType(data)

                return runnable_type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | RunnableType | Unset, data)

        runnable_type = _parse_runnable_type(d.pop("runnable_type", UNSET))

        share_data_update = cls(
            data_id=data_id,
            runnable_type=runnable_type,
        )

        share_data_update.additional_properties = d
        return share_data_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ImagesResponseSizeType0(str, Enum):
    VALUE_0 = "1024x1024"
    VALUE_1 = "1024x1536"
    VALUE_2 = "1536x1024"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class LlmChatOpenaiConfigReasoningEffortType0(str, Enum):
    HIGH = "high"
    LOW = "low"
    MEDIUM = "medium"
    MINIMAL = "minimal"
    NONE = "none"
    XHIGH = "xhigh"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class CronScheduleDayOfWeekType1(str, Enum):
    FRI = "fri"
    MON = "mon"
    SAT = "sat"
    SUN = "sun"
    THU = "thu"
    TUE = "tue"
    WED = "wed"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.metric_type import MetricType
from ..types import UNSET, Unset

T = TypeVar("T", bound="DashboardMetric")


@_attrs_define
class DashboardMetric:
    """Individual metric data point

    Attributes:
        metric_type (MetricType):
        value (float):
        unit (str | Unset):  Default: ''.
        percentage_change (float | None | Unset):
        trend (None | str | Unset):
    """

    metric_type: MetricType
    value: float
    unit: str | Unset = ""
    percentage_change: float | None | Unset = UNSET
    trend: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        metric_type = self.metric_type.value

        value = self.value

        unit = self.unit

        percentage_change: float | None | Unset
        if isinstance(self.percentage_change, Unset):
            percentage_change = UNSET
        else:
            percentage_change = self.percentage_change

        trend: None | str | Unset
        if isinstance(self.trend, Unset):
            trend = UNSET
        else:
            trend = self.trend

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "metric_type": metric_type,
                "value": value,
            }
        )
        if unit is not UNSET:
            field_dict["unit"] = unit
        if percentage_change is not UNSET:
            field_dict["percentage_change"] = percentage_change
        if trend is not UNSET:
            field_dict["trend"] = trend

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        metric_type = MetricType(d.pop("metric_type"))

        value = d.pop("value")

        unit = d.pop("unit", UNSET)

        def _parse_percentage_change(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        percentage_change = _parse_percentage_change(d.pop("percentage_change", UNSET))

        def _parse_trend(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        trend = _parse_trend(d.pop("trend", UNSET))

        dashboard_metric = cls(
            metric_type=metric_type,
            value=value,
            unit=unit,
            percentage_change=percentage_change,
            trend=trend,
        )

        dashboard_metric.additional_properties = d
        return dashboard_metric

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.web_search_options_search_context_size import WebSearchOptionsSearchContextSize
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.web_search_options_user_location import WebSearchOptionsUserLocation


T = TypeVar("T", bound="WebSearchOptions")


@_attrs_define
class WebSearchOptions:
    """This tool searches the web for relevant results to use in a response.
    Learn more about the [web search tool](https://platform.openai.com/docs/guides/tools-web-search?api-mode=chat).

        Attributes:
            search_context_size (WebSearchOptionsSearchContextSize | Unset):
            user_location (None | Unset | WebSearchOptionsUserLocation):
    """

    search_context_size: WebSearchOptionsSearchContextSize | Unset = UNSET
    user_location: None | Unset | WebSearchOptionsUserLocation = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.web_search_options_user_location import WebSearchOptionsUserLocation

        search_context_size: str | Unset = UNSET
        if not isinstance(self.search_context_size, Unset):
            search_context_size = self.search_context_size.value

        user_location: dict[str, Any] | None | Unset
        if isinstance(self.user_location, Unset):
            user_location = UNSET
        elif isinstance(self.user_location, WebSearchOptionsUserLocation):
            user_location = self.user_location.to_dict()
        else:
            user_location = self.user_location

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if search_context_size is not UNSET:
            field_dict["search_context_size"] = search_context_size
        if user_location is not UNSET:
            field_dict["user_location"] = user_location

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.web_search_options_user_location import WebSearchOptionsUserLocation

        d = dict(src_dict)
        _search_context_size = d.pop("search_context_size", UNSET)
        search_context_size: WebSearchOptionsSearchContextSize | Unset
        if isinstance(_search_context_size, Unset):
            search_context_size = UNSET
        else:
            search_context_size = WebSearchOptionsSearchContextSize(_search_context_size)

        def _parse_user_location(data: object) -> None | Unset | WebSearchOptionsUserLocation:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                user_location_type_0 = WebSearchOptionsUserLocation.from_dict(data)

                return user_location_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | Unset | WebSearchOptionsUserLocation, data)

        user_location = _parse_user_location(d.pop("user_location", UNSET))

        web_search_options = cls(
            search_context_size=search_context_size,
            user_location=user_location,
        )

        web_search_options.additional_properties = d
        return web_search_options

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_update_config_type_0 import ActionUpdateConfigType0


T = TypeVar("T", bound="ActionUpdate")


@_attrs_define
class ActionUpdate:
    """
    Attributes:
        name (None | str | Unset):
        version (float | None | Unset):
        description (None | str | Unset):
        user_manual (None | str | Unset):
        config (ActionUpdateConfigType0 | None | Unset):
        is_saved (bool | None | Unset):
        is_published (bool | None | Unset):
        shared_with (list[str] | None | Unset):
        time_saved_per_run (float | None | Unset): Time saved per run in hours for dashboard agent calculations
        dollar_cost_per_hour (float | None | Unset): Dollar cost per hour for dashboard agent calculations
    """

    name: None | str | Unset = UNSET
    version: float | None | Unset = UNSET
    description: None | str | Unset = UNSET
    user_manual: None | str | Unset = UNSET
    config: ActionUpdateConfigType0 | None | Unset = UNSET
    is_saved: bool | None | Unset = UNSET
    is_published: bool | None | Unset = UNSET
    shared_with: list[str] | None | Unset = UNSET
    time_saved_per_run: float | None | Unset = UNSET
    dollar_cost_per_hour: float | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.action_update_config_type_0 import ActionUpdateConfigType0

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        version: float | None | Unset
        if isinstance(self.version, Unset):
            version = UNSET
        else:
            version = self.version

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        user_manual: None | str | Unset
        if isinstance(self.user_manual, Unset):
            user_manual = UNSET
        else:
            user_manual = self.user_manual

        config: dict[str, Any] | None | Unset
        if isinstance(self.config, Unset):
            config = UNSET
        elif isinstance(self.config, ActionUpdateConfigType0):
            config = self.config.to_dict()
        else:
            config = self.config

        is_saved: bool | None | Unset
        if isinstance(self.is_saved, Unset):
            is_saved = UNSET
        else:
            is_saved = self.is_saved

        is_published: bool | None | Unset
        if isinstance(self.is_published, Unset):
            is_published = UNSET
        else:
            is_published = self.is_published

        shared_with: list[str] | None | Unset
        if isinstance(self.shared_with, Unset):
            shared_with = UNSET
        elif isinstance(self.shared_with, list):
            shared_with = self.shared_with

        else:
            shared_with = self.shared_with

        time_saved_per_run: float | None | Unset
        if isinstance(self.time_saved_per_run, Unset):
            time_saved_per_run = UNSET
        else:
            time_saved_per_run = self.time_saved_per_run

        dollar_cost_per_hour: float | None | Unset
        if isinstance(self.dollar_cost_per_hour, Unset):
            dollar_cost_per_hour = UNSET
        else:
            dollar_cost_per_hour = self.dollar_cost_per_hour

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if version is not UNSET:
            field_dict["version"] = version
        if description is not UNSET:
            field_dict["description"] = description
        if user_manual is not UNSET:
            field_dict["user_manual"] = user_manual
        if config is not UNSET:
            field_dict["config"] = config
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if is_published is not UNSET:
            field_dict["is_published"] = is_published
        if shared_with is not UNSET:
            field_dict["shared_with"] = shared_with
        if time_saved_per_run is not UNSET:
            field_dict["time_saved_per_run"] = time_saved_per_run
        if dollar_cost_per_hour is not UNSET:
            field_dict["dollar_cost_per_hour"] = dollar_cost_per_hour

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_update_config_type_0 import ActionUpdateConfigType0

        d = dict(src_dict)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_version(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        version = _parse_version(d.pop("version", UNSET))

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_user_manual(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        user_manual = _parse_user_manual(d.pop("user_manual", UNSET))

        def _parse_config(data: object) -> ActionUpdateConfigType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                config_type_0 = ActionUpdateConfigType0.from_dict(data)

                return config_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionUpdateConfigType0 | None | Unset, data)

        config = _parse_config(d.pop("config", UNSET))

        def _parse_is_saved(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_saved = _parse_is_saved(d.pop("is_saved", UNSET))

        def _parse_is_published(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_published = _parse_is_published(d.pop("is_published", UNSET))

        def _parse_shared_with(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                shared_with_type_0 = cast(list[str], data)

                return shared_with_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        shared_with = _parse_shared_with(d.pop("shared_with", UNSET))

        def _parse_time_saved_per_run(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        time_saved_per_run = _parse_time_saved_per_run(d.pop("time_saved_per_run", UNSET))

        def _parse_dollar_cost_per_hour(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        dollar_cost_per_hour = _parse_dollar_cost_per_hour(d.pop("dollar_cost_per_hour", UNSET))

        action_update = cls(
            name=name,
            version=version,
            description=description,
            user_manual=user_manual,
            config=config,
            is_saved=is_saved,
            is_published=is_published,
            shared_with=shared_with,
            time_saved_per_run=time_saved_per_run,
            dollar_cost_per_hour=dollar_cost_per_hour,
        )

        action_update.additional_properties = d
        return action_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="ActionGraphLiteDoc")


@_attrs_define
class ActionGraphLiteDoc:
    """
    Attributes:
        field_id (str):
        name (str):
        user_id (str):
        created_at (datetime.datetime):
        updated_at (datetime.datetime):
        version (float | Unset):  Default: 0.0.
        description (str | Unset):  Default: ''.
        user_manual (str | Unset):  Default: ''.
        is_saved (bool | Unset):  Default: False.
        is_published (bool | Unset):  Default: False.
        shared_with (list[str] | Unset):
        time_saved_per_file (float | None | Unset):  Default: 0.1.
        dollar_cost_per_hour (float | None | Unset):  Default: 50.0.
    """

    field_id: str
    name: str
    user_id: str
    created_at: datetime.datetime
    updated_at: datetime.datetime
    version: float | Unset = 0.0
    description: str | Unset = ""
    user_manual: str | Unset = ""
    is_saved: bool | Unset = False
    is_published: bool | Unset = False
    shared_with: list[str] | Unset = UNSET
    time_saved_per_file: float | None | Unset = 0.1
    dollar_cost_per_hour: float | None | Unset = 50.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        field_id = self.field_id

        name = self.name

        user_id = self.user_id

        created_at = self.created_at.isoformat()

        updated_at = self.updated_at.isoformat()

        version = self.version

        description = self.description

        user_manual = self.user_manual

        is_saved = self.is_saved

        is_published = self.is_published

        shared_with: list[str] | Unset = UNSET
        if not isinstance(self.shared_with, Unset):
            shared_with = self.shared_with

        time_saved_per_file: float | None | Unset
        if isinstance(self.time_saved_per_file, Unset):
            time_saved_per_file = UNSET
        else:
            time_saved_per_file = self.time_saved_per_file

        dollar_cost_per_hour: float | None | Unset
        if isinstance(self.dollar_cost_per_hour, Unset):
            dollar_cost_per_hour = UNSET
        else:
            dollar_cost_per_hour = self.dollar_cost_per_hour

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "_id": field_id,
                "name": name,
                "user_id": user_id,
                "created_at": created_at,
                "updated_at": updated_at,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version
        if description is not UNSET:
            field_dict["description"] = description
        if user_manual is not UNSET:
            field_dict["user_manual"] = user_manual
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if is_published is not UNSET:
            field_dict["is_published"] = is_published
        if shared_with is not UNSET:
            field_dict["shared_with"] = shared_with
        if time_saved_per_file is not UNSET:
            field_dict["time_saved_per_file"] = time_saved_per_file
        if dollar_cost_per_hour is not UNSET:
            field_dict["dollar_cost_per_hour"] = dollar_cost_per_hour

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        field_id = d.pop("_id")

        name = d.pop("name")

        user_id = d.pop("user_id")

        created_at = isoparse(d.pop("created_at"))

        updated_at = isoparse(d.pop("updated_at"))

        version = d.pop("version", UNSET)

        description = d.pop("description", UNSET)

        user_manual = d.pop("user_manual", UNSET)

        is_saved = d.pop("is_saved", UNSET)

        is_published = d.pop("is_published", UNSET)

        shared_with = cast(list[str], d.pop("shared_with", UNSET))

        def _parse_time_saved_per_file(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        time_saved_per_file = _parse_time_saved_per_file(d.pop("time_saved_per_file", UNSET))

        def _parse_dollar_cost_per_hour(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        dollar_cost_per_hour = _parse_dollar_cost_per_hour(d.pop("dollar_cost_per_hour", UNSET))

        action_graph_lite_doc = cls(
            field_id=field_id,
            name=name,
            user_id=user_id,
            created_at=created_at,
            updated_at=updated_at,
            version=version,
            description=description,
            user_manual=user_manual,
            is_saved=is_saved,
            is_published=is_published,
            shared_with=shared_with,
            time_saved_per_file=time_saved_per_file,
            dollar_cost_per_hour=dollar_cost_per_hour,
        )

        action_graph_lite_doc.additional_properties = d
        return action_graph_lite_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.dashboard_request import DashboardRequest
from ...models.dashboard_response import DashboardResponse
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    body: DashboardRequest,
    force_refresh: bool | Unset = False,
) -> dict[str, Any]:
    headers: dict[str, Any] = {}

    params: dict[str, Any] = {}

    params["force_refresh"] = force_refresh

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "post",
        "url": "/v1/dashboard/",
        "params": params,
    }

    _kwargs["json"] = body.to_dict()

    headers["Content-Type"] = "application/json"

    _kwargs["headers"] = headers
    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> DashboardResponse | HTTPValidationError | None:
    if response.status_code == 200:
        response_200 = DashboardResponse.from_dict(response.json())

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[DashboardResponse | HTTPValidationError]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    body: DashboardRequest,
    force_refresh: bool | Unset = False,
) -> Response[DashboardResponse | HTTPValidationError]:
    """Get Dashboard With Custom Filters

     Get dashboard data with custom filters and date ranges.

    Allows for more complex filtering including custom date ranges.
    Data is served from MongoDB cache for faster response times.

    Args:
        force_refresh (bool | Unset): Force refresh cache Default: False.
        body (DashboardRequest): Request model for dashboard data

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DashboardResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        body=body,
        force_refresh=force_refresh,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    body: DashboardRequest,
    force_refresh: bool | Unset = False,
) -> DashboardResponse | HTTPValidationError | None:
    """Get Dashboard With Custom Filters

     Get dashboard data with custom filters and date ranges.

    Allows for more complex filtering including custom date ranges.
    Data is served from MongoDB cache for faster response times.

    Args:
        force_refresh (bool | Unset): Force refresh cache Default: False.
        body (DashboardRequest): Request model for dashboard data

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DashboardResponse | HTTPValidationError
    """

    return sync_detailed(
        client=client,
        body=body,
        force_refresh=force_refresh,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    body: DashboardRequest,
    force_refresh: bool | Unset = False,
) -> Response[DashboardResponse | HTTPValidationError]:
    """Get Dashboard With Custom Filters

     Get dashboard data with custom filters and date ranges.

    Allows for more complex filtering including custom date ranges.
    Data is served from MongoDB cache for faster response times.

    Args:
        force_refresh (bool | Unset): Force refresh cache Default: False.
        body (DashboardRequest): Request model for dashboard data

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[DashboardResponse | HTTPValidationError]
    """

    kwargs = _get_kwargs(
        body=body,
        force_refresh=force_refresh,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    body: DashboardRequest,
    force_refresh: bool | Unset = False,
) -> DashboardResponse | HTTPValidationError | None:
    """Get Dashboard With Custom Filters

     Get dashboard data with custom filters and date ranges.

    Allows for more complex filtering including custom date ranges.
    Data is served from MongoDB cache for faster response times.

    Args:
        force_refresh (bool | Unset): Force refresh cache Default: False.
        body (DashboardRequest): Request model for dashboard data

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        DashboardResponse | HTTPValidationError
    """

    return (
        await asyncio_detailed(
            client=client,
            body=body,
            force_refresh=force_refresh,
        )
    ).parsed



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.action_type import ActionType
from ..models.data_flow_type import DataFlowType
from ..types import UNSET, Unset

T = TypeVar("T", bound="NodeData")


@_attrs_define
class NodeData:
    """
    Attributes:
        label (str):
        action_id (str):
        type_ (ActionType | None | Unset):
        updated_at (datetime.datetime | None | Unset):
        character_id (None | str | Unset):
        data_flow_type (DataFlowType | Unset):
        user_review_required (bool | Unset):  Default: False.
        user_review_done (bool | Unset):  Default: False.
        user_input_required (bool | Unset):  Default: False.
        user_input_done (bool | Unset):  Default: False.
        is_loop (bool | Unset):  Default: False.
        seperator (str | Unset):  Default: '|'.
        is_split (bool | Unset):  Default: False.
        file_split (bool | Unset):  Default: False.
        api_loop (bool | Unset):  Default: False.
        excel_loop (bool | Unset):  Default: False.
        header (bool | Unset):  Default: False.
        data_processing (bool | Unset):  Default: False.
        file_processing (bool | Unset):  Default: False.
        file_column_index (int | Unset):  Default: 0.
        instruction_column_index (int | Unset):  Default: 0.
        batches (int | Unset):  Default: 0.
        process_url_in_text (bool | Unset):  Default: True.
        process_attachment_url (bool | Unset):  Default: True.
        attachment_requested (int | Unset):  Default: 0.
        sleep_time (float | Unset): Seconds to sleep before executing the node action. Default: 0.0.
    """

    label: str
    action_id: str
    type_: ActionType | None | Unset = UNSET
    updated_at: datetime.datetime | None | Unset = UNSET
    character_id: None | str | Unset = UNSET
    data_flow_type: DataFlowType | Unset = UNSET
    user_review_required: bool | Unset = False
    user_review_done: bool | Unset = False
    user_input_required: bool | Unset = False
    user_input_done: bool | Unset = False
    is_loop: bool | Unset = False
    seperator: str | Unset = "|"
    is_split: bool | Unset = False
    file_split: bool | Unset = False
    api_loop: bool | Unset = False
    excel_loop: bool | Unset = False
    header: bool | Unset = False
    data_processing: bool | Unset = False
    file_processing: bool | Unset = False
    file_column_index: int | Unset = 0
    instruction_column_index: int | Unset = 0
    batches: int | Unset = 0
    process_url_in_text: bool | Unset = True
    process_attachment_url: bool | Unset = True
    attachment_requested: int | Unset = 0
    sleep_time: float | Unset = 0.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        label = self.label

        action_id = self.action_id

        type_: None | str | Unset
        if isinstance(self.type_, Unset):
            type_ = UNSET
        elif isinstance(self.type_, ActionType):
            type_ = self.type_.value
        else:
            type_ = self.type_

        updated_at: None | str | Unset
        if isinstance(self.updated_at, Unset):
            updated_at = UNSET
        elif isinstance(self.updated_at, datetime.datetime):
            updated_at = self.updated_at.isoformat()
        else:
            updated_at = self.updated_at

        character_id: None | str | Unset
        if isinstance(self.character_id, Unset):
            character_id = UNSET
        else:
            character_id = self.character_id

        data_flow_type: str | Unset = UNSET
        if not isinstance(self.data_flow_type, Unset):
            data_flow_type = self.data_flow_type.value

        user_review_required = self.user_review_required

        user_review_done = self.user_review_done

        user_input_required = self.user_input_required

        user_input_done = self.user_input_done

        is_loop = self.is_loop

        seperator = self.seperator

        is_split = self.is_split

        file_split = self.file_split

        api_loop = self.api_loop

        excel_loop = self.excel_loop

        header = self.header

        data_processing = self.data_processing

        file_processing = self.file_processing

        file_column_index = self.file_column_index

        instruction_column_index = self.instruction_column_index

        batches = self.batches

        process_url_in_text = self.process_url_in_text

        process_attachment_url = self.process_attachment_url

        attachment_requested = self.attachment_requested

        sleep_time = self.sleep_time

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "label": label,
                "actionId": action_id,
            }
        )
        if type_ is not UNSET:
            field_dict["type"] = type_
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if character_id is not UNSET:
            field_dict["characterId"] = character_id
        if data_flow_type is not UNSET:
            field_dict["data_flow_type"] = data_flow_type
        if user_review_required is not UNSET:
            field_dict["user_review_required"] = user_review_required
        if user_review_done is not UNSET:
            field_dict["user_review_done"] = user_review_done
        if user_input_required is not UNSET:
            field_dict["user_input_required"] = user_input_required
        if user_input_done is not UNSET:
            field_dict["user_input_done"] = user_input_done
        if is_loop is not UNSET:
            field_dict["is_loop"] = is_loop
        if seperator is not UNSET:
            field_dict["seperator"] = seperator
        if is_split is not UNSET:
            field_dict["is_split"] = is_split
        if file_split is not UNSET:
            field_dict["file_split"] = file_split
        if api_loop is not UNSET:
            field_dict["api_loop"] = api_loop
        if excel_loop is not UNSET:
            field_dict["excel_loop"] = excel_loop
        if header is not UNSET:
            field_dict["header"] = header
        if data_processing is not UNSET:
            field_dict["data_processing"] = data_processing
        if file_processing is not UNSET:
            field_dict["file_processing"] = file_processing
        if file_column_index is not UNSET:
            field_dict["file_column_index"] = file_column_index
        if instruction_column_index is not UNSET:
            field_dict["instruction_column_index"] = instruction_column_index
        if batches is not UNSET:
            field_dict["batches"] = batches
        if process_url_in_text is not UNSET:
            field_dict["process_url_in_text"] = process_url_in_text
        if process_attachment_url is not UNSET:
            field_dict["process_attachment_url"] = process_attachment_url
        if attachment_requested is not UNSET:
            field_dict["attachment_requested"] = attachment_requested
        if sleep_time is not UNSET:
            field_dict["sleep_time"] = sleep_time

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        label = d.pop("label")

        action_id = d.pop("actionId")

        def _parse_type_(data: object) -> ActionType | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                type_type_0 = ActionType(data)

                return type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionType | None | Unset, data)

        type_ = _parse_type_(d.pop("type", UNSET))

        def _parse_updated_at(data: object) -> datetime.datetime | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                updated_at_type_0 = isoparse(data)

                return updated_at_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(datetime.datetime | None | Unset, data)

        updated_at = _parse_updated_at(d.pop("updated_at", UNSET))

        def _parse_character_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        character_id = _parse_character_id(d.pop("characterId", UNSET))

        _data_flow_type = d.pop("data_flow_type", UNSET)
        data_flow_type: DataFlowType | Unset
        if isinstance(_data_flow_type, Unset):
            data_flow_type = UNSET
        else:
            data_flow_type = DataFlowType(_data_flow_type)

        user_review_required = d.pop("user_review_required", UNSET)

        user_review_done = d.pop("user_review_done", UNSET)

        user_input_required = d.pop("user_input_required", UNSET)

        user_input_done = d.pop("user_input_done", UNSET)

        is_loop = d.pop("is_loop", UNSET)

        seperator = d.pop("seperator", UNSET)

        is_split = d.pop("is_split", UNSET)

        file_split = d.pop("file_split", UNSET)

        api_loop = d.pop("api_loop", UNSET)

        excel_loop = d.pop("excel_loop", UNSET)

        header = d.pop("header", UNSET)

        data_processing = d.pop("data_processing", UNSET)

        file_processing = d.pop("file_processing", UNSET)

        file_column_index = d.pop("file_column_index", UNSET)

        instruction_column_index = d.pop("instruction_column_index", UNSET)

        batches = d.pop("batches", UNSET)

        process_url_in_text = d.pop("process_url_in_text", UNSET)

        process_attachment_url = d.pop("process_attachment_url", UNSET)

        attachment_requested = d.pop("attachment_requested", UNSET)

        sleep_time = d.pop("sleep_time", UNSET)

        node_data = cls(
            label=label,
            action_id=action_id,
            type_=type_,
            updated_at=updated_at,
            character_id=character_id,
            data_flow_type=data_flow_type,
            user_review_required=user_review_required,
            user_review_done=user_review_done,
            user_input_required=user_input_required,
            user_input_done=user_input_done,
            is_loop=is_loop,
            seperator=seperator,
            is_split=is_split,
            file_split=file_split,
            api_loop=api_loop,
            excel_loop=excel_loop,
            header=header,
            data_processing=data_processing,
            file_processing=file_processing,
            file_column_index=file_column_index,
            instruction_column_index=instruction_column_index,
            batches=batches,
            process_url_in_text=process_url_in_text,
            process_attachment_url=process_attachment_url,
            attachment_requested=attachment_requested,
            sleep_time=sleep_time,
        )

        node_data.additional_properties = d
        return node_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.file_search_ranking_options_ranker_type_0 import FileSearchRankingOptionsRankerType0
from ..types import UNSET, Unset

T = TypeVar("T", bound="FileSearchRankingOptions")


@_attrs_define
class FileSearchRankingOptions:
    """The ranking options for the file search.

    If not specified, the file search tool will use the `auto` ranker and a score_threshold of 0.

    See the [file search tool documentation](https://platform.openai.com/docs/assistants/tools/file-search#customizing-
    file-search-settings) for more information.

        Attributes:
            score_threshold (float):
            ranker (FileSearchRankingOptionsRankerType0 | None | Unset):
    """

    score_threshold: float
    ranker: FileSearchRankingOptionsRankerType0 | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        score_threshold = self.score_threshold

        ranker: None | str | Unset
        if isinstance(self.ranker, Unset):
            ranker = UNSET
        elif isinstance(self.ranker, FileSearchRankingOptionsRankerType0):
            ranker = self.ranker.value
        else:
            ranker = self.ranker

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "score_threshold": score_threshold,
            }
        )
        if ranker is not UNSET:
            field_dict["ranker"] = ranker

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        score_threshold = d.pop("score_threshold")

        def _parse_ranker(data: object) -> FileSearchRankingOptionsRankerType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                ranker_type_0 = FileSearchRankingOptionsRankerType0(data)

                return ranker_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FileSearchRankingOptionsRankerType0 | None | Unset, data)

        ranker = _parse_ranker(d.pop("ranker", UNSET))

        file_search_ranking_options = cls(
            score_threshold=score_threshold,
            ranker=ranker,
        )

        file_search_ranking_options.additional_properties = d
        return file_search_ranking_options

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class CostConfigType(str, Enum):
    HOURLY_RATE = "hourly_rate"
    INITIAL_INVESTMENT = "initial_investment"
    REVENUE_CONVERSION_RATE = "revenue_conversion_rate"
    TIME_SAVED_RANGES = "time_saved_ranges"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="Image")


@_attrs_define
class Image:
    """Represents the content or the URL of an image generated by the OpenAI API.

    Attributes:
        b64_json (None | str | Unset):
        revised_prompt (None | str | Unset):
        url (None | str | Unset):
    """

    b64_json: None | str | Unset = UNSET
    revised_prompt: None | str | Unset = UNSET
    url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        b64_json: None | str | Unset
        if isinstance(self.b64_json, Unset):
            b64_json = UNSET
        else:
            b64_json = self.b64_json

        revised_prompt: None | str | Unset
        if isinstance(self.revised_prompt, Unset):
            revised_prompt = UNSET
        else:
            revised_prompt = self.revised_prompt

        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if b64_json is not UNSET:
            field_dict["b64_json"] = b64_json
        if revised_prompt is not UNSET:
            field_dict["revised_prompt"] = revised_prompt
        if url is not UNSET:
            field_dict["url"] = url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_b64_json(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        b64_json = _parse_b64_json(d.pop("b64_json", UNSET))

        def _parse_revised_prompt(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        revised_prompt = _parse_revised_prompt(d.pop("revised_prompt", UNSET))

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        image = cls(
            b64_json=b64_json,
            revised_prompt=revised_prompt,
            url=url,
        )

        image.additional_properties = d
        return image

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class SpeechReqVoice(str, Enum):
    ALLOY = "alloy"
    ECHO = "echo"
    FABLE = "fable"
    NOVA = "nova"
    ONYX = "onyx"
    SHIMMER = "shimmer"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_completion_content_part_text_param import ChatCompletionContentPartTextParam


T = TypeVar("T", bound="ChatCompletionDeveloperMessageParam")


@_attrs_define
class ChatCompletionDeveloperMessageParam:
    """Developer-provided instructions that the model should follow, regardless of
    messages sent by the user. With o1 models and newer, `developer` messages
    replace the previous `system` messages.

        Attributes:
            content (list[ChatCompletionContentPartTextParam] | str):
            role (Literal['developer']):
            name (str | Unset):
    """

    content: list[ChatCompletionContentPartTextParam] | str
    role: Literal["developer"]
    name: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        content: list[dict[str, Any]] | str
        if isinstance(self.content, list):
            content = []
            for content_type_1_item_data in self.content:
                content_type_1_item = content_type_1_item_data.to_dict()
                content.append(content_type_1_item)

        else:
            content = self.content

        role = self.role

        name = self.name

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "content": content,
                "role": role,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.chat_completion_content_part_text_param import ChatCompletionContentPartTextParam

        d = dict(src_dict)

        def _parse_content(data: object) -> list[ChatCompletionContentPartTextParam] | str:
            try:
                if not isinstance(data, list):
                    raise TypeError()
                content_type_1 = []
                _content_type_1 = data
                for content_type_1_item_data in _content_type_1:
                    content_type_1_item = ChatCompletionContentPartTextParam.from_dict(content_type_1_item_data)

                    content_type_1.append(content_type_1_item)

                return content_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[ChatCompletionContentPartTextParam] | str, data)

        content = _parse_content(d.pop("content"))

        role = cast(Literal["developer"], d.pop("role"))
        if role != "developer":
            raise ValueError(f"role must match const 'developer', got '{role}'")

        name = d.pop("name", UNSET)

        chat_completion_developer_message_param = cls(
            content=content,
            role=role,
            name=name,
        )

        chat_completion_developer_message_param.additional_properties = d
        return chat_completion_developer_message_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class MetricType(str, Enum):
    ACTIVE_WORKFLOWS = "active_workflows"
    ADDITIONAL_REVENUE = "additional_revenue"
    DOLLARS_SAVED = "dollars_saved"
    ROI_PERCENTAGE = "roi_percentage"
    SCHEDULED_JOBS = "scheduled_jobs"
    TIME_SAVED = "time_saved"
    TOTAL_AGENTS = "total_agents"
    TOTAL_WORKFLOW_RUNS = "total_workflow_runs"

    def __str__(self) -> str:
        return str(self.value)



from enum import Enum


class PartitionParametersParamsHiResModelNameType0(str, Enum):
    CHIPPER = "chipper"
    DETECTRON2_ONNX = "detectron2_onnx"
    YOLOX = "yolox"
    YOLOX_QUANTIZED = "yolox_quantized"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ReadUrlConfig")


@_attrs_define
class ReadUrlConfig:
    """
    Attributes:
        url (None | str | Unset):
        xpath (str | Unset):  Default: '/html/body'.
        attribute (str | Unset):  Default: ''.
    """

    url: None | str | Unset = UNSET
    xpath: str | Unset = "/html/body"
    attribute: str | Unset = ""
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        url: None | str | Unset
        if isinstance(self.url, Unset):
            url = UNSET
        else:
            url = self.url

        xpath = self.xpath

        attribute = self.attribute

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if url is not UNSET:
            field_dict["url"] = url
        if xpath is not UNSET:
            field_dict["xpath"] = xpath
        if attribute is not UNSET:
            field_dict["attribute"] = attribute

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        url = _parse_url(d.pop("url", UNSET))

        xpath = d.pop("xpath", UNSET)

        attribute = d.pop("attribute", UNSET)

        read_url_config = cls(
            url=url,
            xpath=xpath,
            attribute=attribute,
        )

        read_url_config.additional_properties = d
        return read_url_config

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.file_and_processing_params import FileAndProcessingParams


T = TypeVar("T", bound="FilesAndProcessingParams")


@_attrs_define
class FilesAndProcessingParams:
    """
    Attributes:
        files_and_params (list[FileAndProcessingParams]):
    """

    files_and_params: list[FileAndProcessingParams]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        files_and_params = []
        for files_and_params_item_data in self.files_and_params:
            files_and_params_item = files_and_params_item_data.to_dict()
            files_and_params.append(files_and_params_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "files_and_params": files_and_params,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_and_processing_params import FileAndProcessingParams

        d = dict(src_dict)
        files_and_params = []
        _files_and_params = d.pop("files_and_params")
        for files_and_params_item_data in _files_and_params:
            files_and_params_item = FileAndProcessingParams.from_dict(files_and_params_item_data)

            files_and_params.append(files_and_params_item)

        files_and_processing_params = cls(
            files_and_params=files_and_params,
        )

        files_and_processing_params.additional_properties = d
        return files_and_processing_params

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

import datetime
from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

from ..models.app_types import AppTypes
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.app_creds_doc_creds import AppCredsDocCreds


T = TypeVar("T", bound="AppCredsDoc")


@_attrs_define
class AppCredsDoc:
    """
    Attributes:
        app_name (AppTypes | str):
        creds (AppCredsDocCreds):
        user_id (str):
        field_id (str):
        updated_at (datetime.datetime | Unset):
        created_at (datetime.datetime | Unset):
    """

    app_name: AppTypes | str
    creds: AppCredsDocCreds
    user_id: str
    field_id: str
    updated_at: datetime.datetime | Unset = UNSET
    created_at: datetime.datetime | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        app_name: str
        if isinstance(self.app_name, AppTypes):
            app_name = self.app_name.value
        else:
            app_name = self.app_name

        creds = self.creds.to_dict()

        user_id = self.user_id

        field_id = self.field_id

        updated_at: str | Unset = UNSET
        if not isinstance(self.updated_at, Unset):
            updated_at = self.updated_at.isoformat()

        created_at: str | Unset = UNSET
        if not isinstance(self.created_at, Unset):
            created_at = self.created_at.isoformat()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "app_name": app_name,
                "creds": creds,
                "user_id": user_id,
                "_id": field_id,
            }
        )
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if created_at is not UNSET:
            field_dict["created_at"] = created_at

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.app_creds_doc_creds import AppCredsDocCreds

        d = dict(src_dict)

        def _parse_app_name(data: object) -> AppTypes | str:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                app_name_type_1 = AppTypes(data)

                return app_name_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AppTypes | str, data)

        app_name = _parse_app_name(d.pop("app_name"))

        creds = AppCredsDocCreds.from_dict(d.pop("creds"))

        user_id = d.pop("user_id")

        field_id = d.pop("_id")

        _updated_at = d.pop("updated_at", UNSET)
        updated_at: datetime.datetime | Unset
        if isinstance(_updated_at, Unset):
            updated_at = UNSET
        else:
            updated_at = isoparse(_updated_at)

        _created_at = d.pop("created_at", UNSET)
        created_at: datetime.datetime | Unset
        if isinstance(_created_at, Unset):
            created_at = UNSET
        else:
            created_at = isoparse(_created_at)

        app_creds_doc = cls(
            app_name=app_name,
            creds=creds,
            user_id=user_id,
            field_id=field_id,
            updated_at=updated_at,
            created_at=created_at,
        )

        app_creds_doc.additional_properties = d
        return app_creds_doc

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ListIngestedFilesProxyV1IngestionFilesGetScopeType0(str, Enum):
    ALL = "all"
    KB = "kb"
    PERSONAL = "personal"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.app_types import AppTypes

if TYPE_CHECKING:
    from ..models.app_creds_create_creds import AppCredsCreateCreds


T = TypeVar("T", bound="AppCredsCreate")


@_attrs_define
class AppCredsCreate:
    """
    Attributes:
        app_name (AppTypes | str):
        creds (AppCredsCreateCreds):
    """

    app_name: AppTypes | str
    creds: AppCredsCreateCreds
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        app_name: str
        if isinstance(self.app_name, AppTypes):
            app_name = self.app_name.value
        else:
            app_name = self.app_name

        creds = self.creds.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "app_name": app_name,
                "creds": creds,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.app_creds_create_creds import AppCredsCreateCreds

        d = dict(src_dict)

        def _parse_app_name(data: object) -> AppTypes | str:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                app_name_type_1 = AppTypes(data)

                return app_name_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(AppTypes | str, data)

        app_name = _parse_app_name(d.pop("app_name"))

        creds = AppCredsCreateCreds.from_dict(d.pop("creds"))

        app_creds_create = cls(
            app_name=app_name,
            creds=creds,
        )

        app_creds_create.additional_properties = d
        return app_creds_create

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class KnowledgeBaseRole(str, Enum):
    ADMIN = "admin"
    READ = "read"
    READ_WRITE = "read_write"
    READ_WRITE_UPDATE = "read_write_update"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="FileCounts")


@_attrs_define
class FileCounts:
    """
    Attributes:
        cancelled (int):
        completed (int):
        failed (int):
        in_progress (int):
        total (int):
    """

    cancelled: int
    completed: int
    failed: int
    in_progress: int
    total: int
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        cancelled = self.cancelled

        completed = self.completed

        failed = self.failed

        in_progress = self.in_progress

        total = self.total

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "cancelled": cancelled,
                "completed": completed,
                "failed": failed,
                "in_progress": in_progress,
                "total": total,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        cancelled = d.pop("cancelled")

        completed = d.pop("completed")

        failed = d.pop("failed")

        in_progress = d.pop("in_progress")

        total = d.pop("total")

        file_counts = cls(
            cancelled=cancelled,
            completed=completed,
            failed=failed,
            in_progress=in_progress,
            total=total,
        )

        file_counts.additional_properties = d
        return file_counts

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.agent_data_openai_provider import AgentDataOpenaiProvider
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
    from ..models.chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
    from ..models.chat_completion_function_message_param import ChatCompletionFunctionMessageParam
    from ..models.chat_completion_system_message_param import ChatCompletionSystemMessageParam
    from ..models.chat_completion_tool_message_param import ChatCompletionToolMessageParam
    from ..models.chat_completion_user_message_param import ChatCompletionUserMessageParam


T = TypeVar("T", bound="AgentData")


@_attrs_define
class AgentData:
    """
    Attributes:
        openai_provider (AgentDataOpenaiProvider):
        goal (str):
        context (list[ChatCompletionAssistantMessageParam | ChatCompletionDeveloperMessageParam |
            ChatCompletionFunctionMessageParam | ChatCompletionSystemMessageParam | ChatCompletionToolMessageParam |
            ChatCompletionUserMessageParam] | Unset):
    """

    openai_provider: AgentDataOpenaiProvider
    goal: str
    context: (
        list[
            ChatCompletionAssistantMessageParam
            | ChatCompletionDeveloperMessageParam
            | ChatCompletionFunctionMessageParam
            | ChatCompletionSystemMessageParam
            | ChatCompletionToolMessageParam
            | ChatCompletionUserMessageParam
        ]
        | Unset
    ) = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
        from ..models.chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
        from ..models.chat_completion_system_message_param import ChatCompletionSystemMessageParam
        from ..models.chat_completion_tool_message_param import ChatCompletionToolMessageParam
        from ..models.chat_completion_user_message_param import ChatCompletionUserMessageParam

        openai_provider = self.openai_provider.value

        goal = self.goal

        context: list[dict[str, Any]] | Unset = UNSET
        if not isinstance(self.context, Unset):
            context = []
            for context_item_data in self.context:
                context_item: dict[str, Any]
                if isinstance(context_item_data, ChatCompletionDeveloperMessageParam):
                    context_item = context_item_data.to_dict()
                elif isinstance(context_item_data, ChatCompletionSystemMessageParam):
                    context_item = context_item_data.to_dict()
                elif isinstance(context_item_data, ChatCompletionUserMessageParam):
                    context_item = context_item_data.to_dict()
                elif isinstance(context_item_data, ChatCompletionAssistantMessageParam):
                    context_item = context_item_data.to_dict()
                elif isinstance(context_item_data, ChatCompletionToolMessageParam):
                    context_item = context_item_data.to_dict()
                else:
                    context_item = context_item_data.to_dict()

                context.append(context_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "openai_provider": openai_provider,
                "goal": goal,
            }
        )
        if context is not UNSET:
            field_dict["context"] = context

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.chat_completion_assistant_message_param import ChatCompletionAssistantMessageParam
        from ..models.chat_completion_developer_message_param import ChatCompletionDeveloperMessageParam
        from ..models.chat_completion_function_message_param import ChatCompletionFunctionMessageParam
        from ..models.chat_completion_system_message_param import ChatCompletionSystemMessageParam
        from ..models.chat_completion_tool_message_param import ChatCompletionToolMessageParam
        from ..models.chat_completion_user_message_param import ChatCompletionUserMessageParam

        d = dict(src_dict)
        openai_provider = AgentDataOpenaiProvider(d.pop("openai_provider"))

        goal = d.pop("goal")

        _context = d.pop("context", UNSET)
        context: (
            list[
                ChatCompletionAssistantMessageParam
                | ChatCompletionDeveloperMessageParam
                | ChatCompletionFunctionMessageParam
                | ChatCompletionSystemMessageParam
                | ChatCompletionToolMessageParam
                | ChatCompletionUserMessageParam
            ]
            | Unset
        ) = UNSET
        if _context is not UNSET:
            context = []
            for context_item_data in _context:

                def _parse_context_item(
                    data: object,
                ) -> (
                    ChatCompletionAssistantMessageParam
                    | ChatCompletionDeveloperMessageParam
                    | ChatCompletionFunctionMessageParam
                    | ChatCompletionSystemMessageParam
                    | ChatCompletionToolMessageParam
                    | ChatCompletionUserMessageParam
                ):
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        context_item_type_0 = ChatCompletionDeveloperMessageParam.from_dict(data)

                        return context_item_type_0
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        context_item_type_1 = ChatCompletionSystemMessageParam.from_dict(data)

                        return context_item_type_1
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        context_item_type_2 = ChatCompletionUserMessageParam.from_dict(data)

                        return context_item_type_2
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        context_item_type_3 = ChatCompletionAssistantMessageParam.from_dict(data)

                        return context_item_type_3
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    try:
                        if not isinstance(data, dict):
                            raise TypeError()
                        context_item_type_4 = ChatCompletionToolMessageParam.from_dict(data)

                        return context_item_type_4
                    except (TypeError, ValueError, AttributeError, KeyError):
                        pass
                    if not isinstance(data, dict):
                        raise TypeError()
                    context_item_type_5 = ChatCompletionFunctionMessageParam.from_dict(data)

                    return context_item_type_5

                context_item = _parse_context_item(context_item_data)

                context.append(context_item)

        agent_data = cls(
            openai_provider=openai_provider,
            goal=goal,
            context=context,
        )

        agent_data.additional_properties = d
        return agent_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

T = TypeVar("T", bound="BodyRunSlackCmdV1WebhooksSlackCommandPost")


@_attrs_define
class BodyRunSlackCmdV1WebhooksSlackCommandPost:
    """
    Attributes:
        token (str):
        command (str):
        text (str):
        response_url (str):
        trigger_id (str):
        user_id (str):
        team_id (str):
        team_domain (str):
        channel_id (str):
        channel_name (str):
        api_app_id (str):
    """

    token: str
    command: str
    text: str
    response_url: str
    trigger_id: str
    user_id: str
    team_id: str
    team_domain: str
    channel_id: str
    channel_name: str
    api_app_id: str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        token = self.token

        command = self.command

        text = self.text

        response_url = self.response_url

        trigger_id = self.trigger_id

        user_id = self.user_id

        team_id = self.team_id

        team_domain = self.team_domain

        channel_id = self.channel_id

        channel_name = self.channel_name

        api_app_id = self.api_app_id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "token": token,
                "command": command,
                "text": text,
                "response_url": response_url,
                "trigger_id": trigger_id,
                "user_id": user_id,
                "team_id": team_id,
                "team_domain": team_domain,
                "channel_id": channel_id,
                "channel_name": channel_name,
                "api_app_id": api_app_id,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        token = d.pop("token")

        command = d.pop("command")

        text = d.pop("text")

        response_url = d.pop("response_url")

        trigger_id = d.pop("trigger_id")

        user_id = d.pop("user_id")

        team_id = d.pop("team_id")

        team_domain = d.pop("team_domain")

        channel_id = d.pop("channel_id")

        channel_name = d.pop("channel_name")

        api_app_id = d.pop("api_app_id")

        body_run_slack_cmd_v1_webhooks_slack_command_post = cls(
            token=token,
            command=command,
            text=text,
            response_url=response_url,
            trigger_id=trigger_id,
            user_id=user_id,
            team_id=team_id,
            team_domain=team_domain,
            channel_id=channel_id,
            channel_name=channel_name,
            api_app_id=api_app_id,
        )

        body_run_slack_cmd_v1_webhooks_slack_command_post.additional_properties = d
        return body_run_slack_cmd_v1_webhooks_slack_command_post

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.function_definition_parameters_type_0 import FunctionDefinitionParametersType0


T = TypeVar("T", bound="FunctionDefinition")


@_attrs_define
class FunctionDefinition:
    """
    Attributes:
        name (str):
        description (None | str | Unset):
        parameters (FunctionDefinitionParametersType0 | None | Unset):
        strict (bool | None | Unset):
    """

    name: str
    description: None | str | Unset = UNSET
    parameters: FunctionDefinitionParametersType0 | None | Unset = UNSET
    strict: bool | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.function_definition_parameters_type_0 import FunctionDefinitionParametersType0

        name = self.name

        description: None | str | Unset
        if isinstance(self.description, Unset):
            description = UNSET
        else:
            description = self.description

        parameters: dict[str, Any] | None | Unset
        if isinstance(self.parameters, Unset):
            parameters = UNSET
        elif isinstance(self.parameters, FunctionDefinitionParametersType0):
            parameters = self.parameters.to_dict()
        else:
            parameters = self.parameters

        strict: bool | None | Unset
        if isinstance(self.strict, Unset):
            strict = UNSET
        else:
            strict = self.strict

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if parameters is not UNSET:
            field_dict["parameters"] = parameters
        if strict is not UNSET:
            field_dict["strict"] = strict

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.function_definition_parameters_type_0 import FunctionDefinitionParametersType0

        d = dict(src_dict)
        name = d.pop("name")

        def _parse_description(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        description = _parse_description(d.pop("description", UNSET))

        def _parse_parameters(data: object) -> FunctionDefinitionParametersType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                parameters_type_0 = FunctionDefinitionParametersType0.from_dict(data)

                return parameters_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FunctionDefinitionParametersType0 | None | Unset, data)

        parameters = _parse_parameters(d.pop("parameters", UNSET))

        def _parse_strict(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        strict = _parse_strict(d.pop("strict", UNSET))

        function_definition = cls(
            name=name,
            description=description,
            parameters=parameters,
            strict=strict,
        )

        function_definition.additional_properties = d
        return function_definition

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class FileObjectPurpose(str, Enum):
    ASSISTANTS = "assistants"
    ASSISTANTS_OUTPUT = "assistants_output"
    BATCH = "batch"
    BATCH_OUTPUT = "batch_output"
    FINE_TUNE = "fine-tune"
    FINE_TUNE_RESULTS = "fine-tune-results"
    USER_DATA = "user_data"
    VISION = "vision"

    def __str__(self) -> str:
        return str(self.value)



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.job_status_schema import JobStatusSchema


T = TypeVar("T", bound="FileListResponseSchema")


@_attrs_define
class FileListResponseSchema:
    """
    Attributes:
        total (int):
        limit (int):
        offset (int):
        files (list[JobStatusSchema]):
    """

    total: int
    limit: int
    offset: int
    files: list[JobStatusSchema]
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        total = self.total

        limit = self.limit

        offset = self.offset

        files = []
        for files_item_data in self.files:
            files_item = files_item_data.to_dict()
            files.append(files_item)

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "total": total,
                "limit": limit,
                "offset": offset,
                "files": files,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.job_status_schema import JobStatusSchema

        d = dict(src_dict)
        total = d.pop("total")

        limit = d.pop("limit")

        offset = d.pop("offset")

        files = []
        _files = d.pop("files")
        for files_item_data in _files:
            files_item = JobStatusSchema.from_dict(files_item_data)

            files.append(files_item)

        file_list_response_schema = cls(
            total=total,
            limit=limit,
            offset=offset,
            files=files,
        )

        file_list_response_schema.additional_properties = d
        return file_list_response_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.tool_resources_code_interpreter import ToolResourcesCodeInterpreter
    from ..models.tool_resources_file_search import ToolResourcesFileSearch


T = TypeVar("T", bound="ToolResources")


@_attrs_define
class ToolResources:
    """A set of resources that are used by the assistant's tools.

    The resources are specific to the type of tool. For example, the `code_interpreter` tool requires a list of file
    IDs, while the `file_search` tool requires a list of vector store IDs.

        Attributes:
            code_interpreter (None | ToolResourcesCodeInterpreter | Unset):
            file_search (None | ToolResourcesFileSearch | Unset):
    """

    code_interpreter: None | ToolResourcesCodeInterpreter | Unset = UNSET
    file_search: None | ToolResourcesFileSearch | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.tool_resources_code_interpreter import ToolResourcesCodeInterpreter
        from ..models.tool_resources_file_search import ToolResourcesFileSearch

        code_interpreter: dict[str, Any] | None | Unset
        if isinstance(self.code_interpreter, Unset):
            code_interpreter = UNSET
        elif isinstance(self.code_interpreter, ToolResourcesCodeInterpreter):
            code_interpreter = self.code_interpreter.to_dict()
        else:
            code_interpreter = self.code_interpreter

        file_search: dict[str, Any] | None | Unset
        if isinstance(self.file_search, Unset):
            file_search = UNSET
        elif isinstance(self.file_search, ToolResourcesFileSearch):
            file_search = self.file_search.to_dict()
        else:
            file_search = self.file_search

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if code_interpreter is not UNSET:
            field_dict["code_interpreter"] = code_interpreter
        if file_search is not UNSET:
            field_dict["file_search"] = file_search

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.tool_resources_code_interpreter import ToolResourcesCodeInterpreter
        from ..models.tool_resources_file_search import ToolResourcesFileSearch

        d = dict(src_dict)

        def _parse_code_interpreter(data: object) -> None | ToolResourcesCodeInterpreter | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                code_interpreter_type_0 = ToolResourcesCodeInterpreter.from_dict(data)

                return code_interpreter_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ToolResourcesCodeInterpreter | Unset, data)

        code_interpreter = _parse_code_interpreter(d.pop("code_interpreter", UNSET))

        def _parse_file_search(data: object) -> None | ToolResourcesFileSearch | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                file_search_type_0 = ToolResourcesFileSearch.from_dict(data)

                return file_search_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | ToolResourcesFileSearch | Unset, data)

        file_search = _parse_file_search(d.pop("file_search", UNSET))

        tool_resources = cls(
            code_interpreter=code_interpreter,
            file_search=file_search,
        )

        tool_resources.additional_properties = d
        return tool_resources

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.file_search_ranking_options import FileSearchRankingOptions


T = TypeVar("T", bound="FileSearch")


@_attrs_define
class FileSearch:
    """Overrides for the file search tool.

    Attributes:
        max_num_results (int | None | Unset):
        ranking_options (FileSearchRankingOptions | None | Unset):
    """

    max_num_results: int | None | Unset = UNSET
    ranking_options: FileSearchRankingOptions | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.file_search_ranking_options import FileSearchRankingOptions

        max_num_results: int | None | Unset
        if isinstance(self.max_num_results, Unset):
            max_num_results = UNSET
        else:
            max_num_results = self.max_num_results

        ranking_options: dict[str, Any] | None | Unset
        if isinstance(self.ranking_options, Unset):
            ranking_options = UNSET
        elif isinstance(self.ranking_options, FileSearchRankingOptions):
            ranking_options = self.ranking_options.to_dict()
        else:
            ranking_options = self.ranking_options

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if max_num_results is not UNSET:
            field_dict["max_num_results"] = max_num_results
        if ranking_options is not UNSET:
            field_dict["ranking_options"] = ranking_options

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.file_search_ranking_options import FileSearchRankingOptions

        d = dict(src_dict)

        def _parse_max_num_results(data: object) -> int | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(int | None | Unset, data)

        max_num_results = _parse_max_num_results(d.pop("max_num_results", UNSET))

        def _parse_ranking_options(data: object) -> FileSearchRankingOptions | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                ranking_options_type_0 = FileSearchRankingOptions.from_dict(data)

                return ranking_options_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(FileSearchRankingOptions | None | Unset, data)

        ranking_options = _parse_ranking_options(d.pop("ranking_options", UNSET))

        file_search = cls(
            max_num_results=max_num_results,
            ranking_options=ranking_options,
        )

        file_search.additional_properties = d
        return file_search

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.action_type import ActionType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_result import ActionResult
    from ..models.data_block import DataBlock
    from ..models.data_blocks import DataBlocks
    from ..models.translation_req import TranslationReq


T = TypeVar("T", bound="ActionCreateAudio2TextTranslationOpenai")


@_attrs_define
class ActionCreateAudio2TextTranslationOpenai:
    """
    Attributes:
        name (str):
        config (TranslationReq):
        version (float | Unset):  Default: 0.0.
        description (str | Unset):  Default: ''.
        user_manual (str | Unset):  Default: ''.
        type_ (ActionType | Unset):
        is_saved (bool | Unset):  Default: False.
        results (list[ActionResult] | None | Unset):
        input_ (DataBlock | None | Unset):
        output (DataBlocks | None | Unset):
        is_published (bool | Unset):  Default: False.
        shared_with (list[str] | Unset):
        time_saved_per_run (float | None | Unset): Time saved per run in hours for dashboard agent calculations Default:
            0.5.
        dollar_cost_per_hour (float | None | Unset): Dollar cost per hour for dashboard agent calculations Default:
            50.0.
    """

    name: str
    config: TranslationReq
    version: float | Unset = 0.0
    description: str | Unset = ""
    user_manual: str | Unset = ""
    type_: ActionType | Unset = UNSET
    is_saved: bool | Unset = False
    results: list[ActionResult] | None | Unset = UNSET
    input_: DataBlock | None | Unset = UNSET
    output: DataBlocks | None | Unset = UNSET
    is_published: bool | Unset = False
    shared_with: list[str] | Unset = UNSET
    time_saved_per_run: float | None | Unset = 0.5
    dollar_cost_per_hour: float | None | Unset = 50.0
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.data_block import DataBlock
        from ..models.data_blocks import DataBlocks

        name = self.name

        config = self.config.to_dict()

        version = self.version

        description = self.description

        user_manual = self.user_manual

        type_: str | Unset = UNSET
        if not isinstance(self.type_, Unset):
            type_ = self.type_.value

        is_saved = self.is_saved

        results: list[dict[str, Any]] | None | Unset
        if isinstance(self.results, Unset):
            results = UNSET
        elif isinstance(self.results, list):
            results = []
            for results_type_0_item_data in self.results:
                results_type_0_item = results_type_0_item_data.to_dict()
                results.append(results_type_0_item)

        else:
            results = self.results

        input_: dict[str, Any] | None | Unset
        if isinstance(self.input_, Unset):
            input_ = UNSET
        elif isinstance(self.input_, DataBlock):
            input_ = self.input_.to_dict()
        else:
            input_ = self.input_

        output: dict[str, Any] | None | Unset
        if isinstance(self.output, Unset):
            output = UNSET
        elif isinstance(self.output, DataBlocks):
            output = self.output.to_dict()
        else:
            output = self.output

        is_published = self.is_published

        shared_with: list[str] | Unset = UNSET
        if not isinstance(self.shared_with, Unset):
            shared_with = self.shared_with

        time_saved_per_run: float | None | Unset
        if isinstance(self.time_saved_per_run, Unset):
            time_saved_per_run = UNSET
        else:
            time_saved_per_run = self.time_saved_per_run

        dollar_cost_per_hour: float | None | Unset
        if isinstance(self.dollar_cost_per_hour, Unset):
            dollar_cost_per_hour = UNSET
        else:
            dollar_cost_per_hour = self.dollar_cost_per_hour

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
                "config": config,
            }
        )
        if version is not UNSET:
            field_dict["version"] = version
        if description is not UNSET:
            field_dict["description"] = description
        if user_manual is not UNSET:
            field_dict["user_manual"] = user_manual
        if type_ is not UNSET:
            field_dict["type"] = type_
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if results is not UNSET:
            field_dict["results"] = results
        if input_ is not UNSET:
            field_dict["input"] = input_
        if output is not UNSET:
            field_dict["output"] = output
        if is_published is not UNSET:
            field_dict["is_published"] = is_published
        if shared_with is not UNSET:
            field_dict["shared_with"] = shared_with
        if time_saved_per_run is not UNSET:
            field_dict["time_saved_per_run"] = time_saved_per_run
        if dollar_cost_per_hour is not UNSET:
            field_dict["dollar_cost_per_hour"] = dollar_cost_per_hour

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_result import ActionResult
        from ..models.data_block import DataBlock
        from ..models.data_blocks import DataBlocks
        from ..models.translation_req import TranslationReq

        d = dict(src_dict)
        name = d.pop("name")

        config = TranslationReq.from_dict(d.pop("config"))

        version = d.pop("version", UNSET)

        description = d.pop("description", UNSET)

        user_manual = d.pop("user_manual", UNSET)

        _type_ = d.pop("type", UNSET)
        type_: ActionType | Unset
        if isinstance(_type_, Unset):
            type_ = UNSET
        else:
            type_ = ActionType(_type_)

        is_saved = d.pop("is_saved", UNSET)

        def _parse_results(data: object) -> list[ActionResult] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                results_type_0 = []
                _results_type_0 = data
                for results_type_0_item_data in _results_type_0:
                    results_type_0_item = ActionResult.from_dict(results_type_0_item_data)

                    results_type_0.append(results_type_0_item)

                return results_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[ActionResult] | None | Unset, data)

        results = _parse_results(d.pop("results", UNSET))

        def _parse_input_(data: object) -> DataBlock | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                input_type_0 = DataBlock.from_dict(data)

                return input_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DataBlock | None | Unset, data)

        input_ = _parse_input_(d.pop("input", UNSET))

        def _parse_output(data: object) -> DataBlocks | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                output_type_0 = DataBlocks.from_dict(data)

                return output_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DataBlocks | None | Unset, data)

        output = _parse_output(d.pop("output", UNSET))

        is_published = d.pop("is_published", UNSET)

        shared_with = cast(list[str], d.pop("shared_with", UNSET))

        def _parse_time_saved_per_run(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        time_saved_per_run = _parse_time_saved_per_run(d.pop("time_saved_per_run", UNSET))

        def _parse_dollar_cost_per_hour(data: object) -> float | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(float | None | Unset, data)

        dollar_cost_per_hour = _parse_dollar_cost_per_hour(d.pop("dollar_cost_per_hour", UNSET))

        action_create_audio_2_text_translation_openai = cls(
            name=name,
            config=config,
            version=version,
            description=description,
            user_manual=user_manual,
            type_=type_,
            is_saved=is_saved,
            results=results,
            input_=input_,
            output=output,
            is_published=is_published,
            shared_with=shared_with,
            time_saved_per_run=time_saved_per_run,
            dollar_cost_per_hour=dollar_cost_per_hour,
        )

        action_create_audio_2_text_translation_openai.additional_properties = d
        return action_create_audio_2_text_translation_openai

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.event_result_status import EventResultStatus
from ..models.event_type import EventType
from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.action_graph_doc import ActionGraphDoc
    from ..models.action_graph_lite_doc import ActionGraphLiteDoc
    from ..models.text_obj import TextObj


T = TypeVar("T", bound="ActionGraphResultUpdate")


@_attrs_define
class ActionGraphResultUpdate:
    """Input from User to update Action Result

    Attributes:
        name (None | str | Unset):
        type_ (EventType | None | Unset):
        status (EventResultStatus | None | Unset):
        error_message (None | TextObj | Unset):
        is_saved (bool | None | Unset):
        result (ActionGraphDoc | ActionGraphLiteDoc | None | Unset):
        file_url (None | str | Unset):
    """

    name: None | str | Unset = UNSET
    type_: EventType | None | Unset = UNSET
    status: EventResultStatus | None | Unset = UNSET
    error_message: None | TextObj | Unset = UNSET
    is_saved: bool | None | Unset = UNSET
    result: ActionGraphDoc | ActionGraphLiteDoc | None | Unset = UNSET
    file_url: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.action_graph_doc import ActionGraphDoc
        from ..models.action_graph_lite_doc import ActionGraphLiteDoc
        from ..models.text_obj import TextObj

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        type_: None | str | Unset
        if isinstance(self.type_, Unset):
            type_ = UNSET
        elif isinstance(self.type_, EventType):
            type_ = self.type_.value
        else:
            type_ = self.type_

        status: None | str | Unset
        if isinstance(self.status, Unset):
            status = UNSET
        elif isinstance(self.status, EventResultStatus):
            status = self.status.value
        else:
            status = self.status

        error_message: dict[str, Any] | None | Unset
        if isinstance(self.error_message, Unset):
            error_message = UNSET
        elif isinstance(self.error_message, TextObj):
            error_message = self.error_message.to_dict()
        else:
            error_message = self.error_message

        is_saved: bool | None | Unset
        if isinstance(self.is_saved, Unset):
            is_saved = UNSET
        else:
            is_saved = self.is_saved

        result: dict[str, Any] | None | Unset
        if isinstance(self.result, Unset):
            result = UNSET
        elif isinstance(self.result, ActionGraphDoc):
            result = self.result.to_dict()
        elif isinstance(self.result, ActionGraphLiteDoc):
            result = self.result.to_dict()
        else:
            result = self.result

        file_url: None | str | Unset
        if isinstance(self.file_url, Unset):
            file_url = UNSET
        else:
            file_url = self.file_url

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if type_ is not UNSET:
            field_dict["type"] = type_
        if status is not UNSET:
            field_dict["status"] = status
        if error_message is not UNSET:
            field_dict["error_message"] = error_message
        if is_saved is not UNSET:
            field_dict["is_saved"] = is_saved
        if result is not UNSET:
            field_dict["result"] = result
        if file_url is not UNSET:
            field_dict["file_url"] = file_url

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.action_graph_doc import ActionGraphDoc
        from ..models.action_graph_lite_doc import ActionGraphLiteDoc
        from ..models.text_obj import TextObj

        d = dict(src_dict)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_type_(data: object) -> EventType | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                type_type_0 = EventType(data)

                return type_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(EventType | None | Unset, data)

        type_ = _parse_type_(d.pop("type", UNSET))

        def _parse_status(data: object) -> EventResultStatus | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, str):
                    raise TypeError()
                status_type_0 = EventResultStatus(data)

                return status_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(EventResultStatus | None | Unset, data)

        status = _parse_status(d.pop("status", UNSET))

        def _parse_error_message(data: object) -> None | TextObj | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                error_message_type_0 = TextObj.from_dict(data)

                return error_message_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(None | TextObj | Unset, data)

        error_message = _parse_error_message(d.pop("error_message", UNSET))

        def _parse_is_saved(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_saved = _parse_is_saved(d.pop("is_saved", UNSET))

        def _parse_result(data: object) -> ActionGraphDoc | ActionGraphLiteDoc | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                result_type_0 = ActionGraphDoc.from_dict(data)

                return result_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                result_type_1 = ActionGraphLiteDoc.from_dict(data)

                return result_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ActionGraphDoc | ActionGraphLiteDoc | None | Unset, data)

        result = _parse_result(d.pop("result", UNSET))

        def _parse_file_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        file_url = _parse_file_url(d.pop("file_url", UNSET))

        action_graph_result_update = cls(
            name=name,
            type_=type_,
            status=status,
            error_message=error_message,
            is_saved=is_saved,
            result=result,
            file_url=file_url,
        )

        action_graph_result_update.additional_properties = d
        return action_graph_result_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="ADPClientCredentialsRequest")


@_attrs_define
class ADPClientCredentialsRequest:
    r""" Request model for ADP client credentials authentication.

    ADP requires Mutual SSL (mTLS) Authentication:
    - client_id: Your ADP client ID from ADP Developer Portal
    - client_secret: Your ADP client secret
    - ssl_cert_path: Path to SSL certificate file (PEM or PFX format)
    - ssl_key_path: Path to private key file (optional if embedded in cert)

    Supported path formats for certificates:
    - Local file path: /path/to/cert.pem
    - S3 URI: s3://bucket-name/path/to/cert.pem
    - S3 HTTPS URL: https://bucket-name.s3.region.amazonaws.com/path/to/cert.pem

    Certificate Setup:
    1. Obtain SSL certificate and private key from:
       - ADP Partners: Developer Self-Service Portal (automated)
       - ADP Clients: ADP Certificate Signing Tool (manual CSR)
       - API Central Users: API Central (automated)

    2. Upload certificates to S3 (recommended) or provide local paths:
       s3://your-bucket/adp/cert.pem
       s3://your-bucket/adp/key.key

    3. Optionally combine into single PFX file:
       openssl pkcs12 -export -out adp.pfx -name "OrgName Mutual SSL" \
           -inkey private-key.key -in public-certificate.pem

    Note: Certificates are valid for 2 years. ADP sends renewal notices
    60 days before expiration.

        Attributes:
            client_id (str):
            client_secret (str):
            ssl_cert_path (str):
            ssl_key_path (None | str | Unset):
     """

    client_id: str
    client_secret: str
    ssl_cert_path: str
    ssl_key_path: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        client_id = self.client_id

        client_secret = self.client_secret

        ssl_cert_path = self.ssl_cert_path

        ssl_key_path: None | str | Unset
        if isinstance(self.ssl_key_path, Unset):
            ssl_key_path = UNSET
        else:
            ssl_key_path = self.ssl_key_path

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "client_id": client_id,
                "client_secret": client_secret,
                "ssl_cert_path": ssl_cert_path,
            }
        )
        if ssl_key_path is not UNSET:
            field_dict["ssl_key_path"] = ssl_key_path

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        client_id = d.pop("client_id")

        client_secret = d.pop("client_secret")

        ssl_cert_path = d.pop("ssl_cert_path")

        def _parse_ssl_key_path(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        ssl_key_path = _parse_ssl_key_path(d.pop("ssl_key_path", UNSET))

        adp_client_credentials_request = cls(
            client_id=client_id,
            client_secret=client_secret,
            ssl_cert_path=ssl_cert_path,
            ssl_key_path=ssl_key_path,
        )

        adp_client_credentials_request.additional_properties = d
        return adp_client_credentials_request

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.data_blocks import DataBlocks


T = TypeVar("T", bound="PromptUpdate")


@_attrs_define
class PromptUpdate:
    """
    Attributes:
        name (None | str | Unset):
        tags (list[str] | None | Unset):
        is_published (bool | None | Unset):
        blocks (DataBlocks | None | Unset):
    """

    name: None | str | Unset = UNSET
    tags: list[str] | None | Unset = UNSET
    is_published: bool | None | Unset = UNSET
    blocks: DataBlocks | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.data_blocks import DataBlocks

        name: None | str | Unset
        if isinstance(self.name, Unset):
            name = UNSET
        else:
            name = self.name

        tags: list[str] | None | Unset
        if isinstance(self.tags, Unset):
            tags = UNSET
        elif isinstance(self.tags, list):
            tags = self.tags

        else:
            tags = self.tags

        is_published: bool | None | Unset
        if isinstance(self.is_published, Unset):
            is_published = UNSET
        else:
            is_published = self.is_published

        blocks: dict[str, Any] | None | Unset
        if isinstance(self.blocks, Unset):
            blocks = UNSET
        elif isinstance(self.blocks, DataBlocks):
            blocks = self.blocks.to_dict()
        else:
            blocks = self.blocks

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if tags is not UNSET:
            field_dict["tags"] = tags
        if is_published is not UNSET:
            field_dict["is_published"] = is_published
        if blocks is not UNSET:
            field_dict["blocks"] = blocks

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.data_blocks import DataBlocks

        d = dict(src_dict)

        def _parse_name(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        name = _parse_name(d.pop("name", UNSET))

        def _parse_tags(data: object) -> list[str] | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, list):
                    raise TypeError()
                tags_type_0 = cast(list[str], data)

                return tags_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(list[str] | None | Unset, data)

        tags = _parse_tags(d.pop("tags", UNSET))

        def _parse_is_published(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        is_published = _parse_is_published(d.pop("is_published", UNSET))

        def _parse_blocks(data: object) -> DataBlocks | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                blocks_type_0 = DataBlocks.from_dict(data)

                return blocks_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(DataBlocks | None | Unset, data)

        blocks = _parse_blocks(d.pop("blocks", UNSET))

        prompt_update = cls(
            name=name,
            tags=tags,
            is_published=is_published,
            blocks=blocks,
        )

        prompt_update.additional_properties = d
        return prompt_update

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="WebSearchOptionsUserLocationApproximate")


@_attrs_define
class WebSearchOptionsUserLocationApproximate:
    """Approximate location parameters for the search.

    Attributes:
        city (str | Unset):
        country (str | Unset):
        region (str | Unset):
        timezone (str | Unset):
    """

    city: str | Unset = UNSET
    country: str | Unset = UNSET
    region: str | Unset = UNSET
    timezone: str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        city = self.city

        country = self.country

        region = self.region

        timezone = self.timezone

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if city is not UNSET:
            field_dict["city"] = city
        if country is not UNSET:
            field_dict["country"] = country
        if region is not UNSET:
            field_dict["region"] = region
        if timezone is not UNSET:
            field_dict["timezone"] = timezone

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        city = d.pop("city", UNSET)

        country = d.pop("country", UNSET)

        region = d.pop("region", UNSET)

        timezone = d.pop("timezone", UNSET)

        web_search_options_user_location_approximate = cls(
            city=city,
            country=country,
            region=region,
            timezone=timezone,
        )

        web_search_options_user_location_approximate.additional_properties = d
        return web_search_options_user_location_approximate

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.graph_rag_search_data_graph_visualization_type_0 import GraphRAGSearchDataGraphVisualizationType0
    from ..models.source_chunk import SourceChunk


T = TypeVar("T", bound="GraphRAGSearchData")


@_attrs_define
class GraphRAGSearchData:
    """
    Attributes:
        sources (list[SourceChunk]):
        graph_visualization (GraphRAGSearchDataGraphVisualizationType0 | None | Unset):
    """

    sources: list[SourceChunk]
    graph_visualization: GraphRAGSearchDataGraphVisualizationType0 | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        from ..models.graph_rag_search_data_graph_visualization_type_0 import GraphRAGSearchDataGraphVisualizationType0

        sources = []
        for sources_item_data in self.sources:
            sources_item = sources_item_data.to_dict()
            sources.append(sources_item)

        graph_visualization: dict[str, Any] | None | Unset
        if isinstance(self.graph_visualization, Unset):
            graph_visualization = UNSET
        elif isinstance(self.graph_visualization, GraphRAGSearchDataGraphVisualizationType0):
            graph_visualization = self.graph_visualization.to_dict()
        else:
            graph_visualization = self.graph_visualization

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sources": sources,
            }
        )
        if graph_visualization is not UNSET:
            field_dict["graph_visualization"] = graph_visualization

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.graph_rag_search_data_graph_visualization_type_0 import GraphRAGSearchDataGraphVisualizationType0
        from ..models.source_chunk import SourceChunk

        d = dict(src_dict)
        sources = []
        _sources = d.pop("sources")
        for sources_item_data in _sources:
            sources_item = SourceChunk.from_dict(sources_item_data)

            sources.append(sources_item)

        def _parse_graph_visualization(data: object) -> GraphRAGSearchDataGraphVisualizationType0 | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            try:
                if not isinstance(data, dict):
                    raise TypeError()
                graph_visualization_type_0 = GraphRAGSearchDataGraphVisualizationType0.from_dict(data)

                return graph_visualization_type_0
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(GraphRAGSearchDataGraphVisualizationType0 | None | Unset, data)

        graph_visualization = _parse_graph_visualization(d.pop("graph_visualization", UNSET))

        graph_rag_search_data = cls(
            sources=sources,
            graph_visualization=graph_visualization,
        )

        graph_rag_search_data.additional_properties = d
        return graph_rag_search_data

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import TYPE_CHECKING, Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.json_schema_schema import JSONSchemaSchema


T = TypeVar("T", bound="JSONSchema")


@_attrs_define
class JSONSchema:
    """Structured Outputs configuration options, including a JSON Schema.

    Attributes:
        name (str):
        description (str | Unset):
        schema (JSONSchemaSchema | Unset):
        strict (bool | None | Unset):
    """

    name: str
    description: str | Unset = UNSET
    schema: JSONSchemaSchema | Unset = UNSET
    strict: bool | None | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        name = self.name

        description = self.description

        schema: dict[str, Any] | Unset = UNSET
        if not isinstance(self.schema, Unset):
            schema = self.schema.to_dict()

        strict: bool | None | Unset
        if isinstance(self.strict, Unset):
            strict = UNSET
        else:
            strict = self.strict

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "name": name,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description
        if schema is not UNSET:
            field_dict["schema"] = schema
        if strict is not UNSET:
            field_dict["strict"] = strict

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.json_schema_schema import JSONSchemaSchema

        d = dict(src_dict)
        name = d.pop("name")

        description = d.pop("description", UNSET)

        _schema = d.pop("schema", UNSET)
        schema: JSONSchemaSchema | Unset
        if isinstance(_schema, Unset):
            schema = UNSET
        else:
            schema = JSONSchemaSchema.from_dict(_schema)

        def _parse_strict(data: object) -> bool | None | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(bool | None | Unset, data)

        strict = _parse_strict(d.pop("strict", UNSET))

        json_schema = cls(
            name=name,
            description=description,
            schema=schema,
            strict=strict,
        )

        json_schema.additional_properties = d
        return json_schema

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, Literal, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.translation_req_response_format import TranslationReqResponseFormat
from ..types import UNSET, Unset

T = TypeVar("T", bound="TranslationReq")


@_attrs_define
class TranslationReq:
    """
    Attributes:
        file_url (None | str | Unset):
        model (Literal['whisper-1'] | Unset):  Default: 'whisper-1'.
        prompt (str | Unset):  Default: ''.
        response_format (TranslationReqResponseFormat | Unset):  Default: TranslationReqResponseFormat.JSON.
        temperature (float | Unset):  Default: 0.2.
    """

    file_url: None | str | Unset = UNSET
    model: Literal["whisper-1"] | Unset = "whisper-1"
    prompt: str | Unset = ""
    response_format: TranslationReqResponseFormat | Unset = TranslationReqResponseFormat.JSON
    temperature: float | Unset = 0.2
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        file_url: None | str | Unset
        if isinstance(self.file_url, Unset):
            file_url = UNSET
        else:
            file_url = self.file_url

        model = self.model

        prompt = self.prompt

        response_format: str | Unset = UNSET
        if not isinstance(self.response_format, Unset):
            response_format = self.response_format.value

        temperature = self.temperature

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if file_url is not UNSET:
            field_dict["file_url"] = file_url
        if model is not UNSET:
            field_dict["model"] = model
        if prompt is not UNSET:
            field_dict["prompt"] = prompt
        if response_format is not UNSET:
            field_dict["response_format"] = response_format
        if temperature is not UNSET:
            field_dict["temperature"] = temperature

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)

        def _parse_file_url(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        file_url = _parse_file_url(d.pop("file_url", UNSET))

        model = cast(Literal["whisper-1"] | Unset, d.pop("model", UNSET))
        if model != "whisper-1" and not isinstance(model, Unset):
            raise ValueError(f"model must match const 'whisper-1', got '{model}'")

        prompt = d.pop("prompt", UNSET)

        _response_format = d.pop("response_format", UNSET)
        response_format: TranslationReqResponseFormat | Unset
        if isinstance(_response_format, Unset):
            response_format = UNSET
        else:
            response_format = TranslationReqResponseFormat(_response_format)

        temperature = d.pop("temperature", UNSET)

        translation_req = cls(
            file_url=file_url,
            model=model,
            prompt=prompt,
            response_format=response_format,
            temperature=temperature,
        )

        translation_req.additional_properties = d
        return translation_req

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, cast

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..models.chat_completion_audio_param_format import ChatCompletionAudioParamFormat
from ..models.chat_completion_audio_param_voice_type_1 import ChatCompletionAudioParamVoiceType1

T = TypeVar("T", bound="ChatCompletionAudioParam")


@_attrs_define
class ChatCompletionAudioParam:
    """Parameters for audio output.

    Required when audio output is requested with
    `modalities: ["audio"]`. [Learn more](https://platform.openai.com/docs/guides/audio).

        Attributes:
            format_ (ChatCompletionAudioParamFormat):
            voice (ChatCompletionAudioParamVoiceType1 | str):
    """

    format_: ChatCompletionAudioParamFormat
    voice: ChatCompletionAudioParamVoiceType1 | str
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        format_ = self.format_.value

        voice: str
        if isinstance(self.voice, ChatCompletionAudioParamVoiceType1):
            voice = self.voice.value
        else:
            voice = self.voice

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "format": format_,
                "voice": voice,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        d = dict(src_dict)
        format_ = ChatCompletionAudioParamFormat(d.pop("format"))

        def _parse_voice(data: object) -> ChatCompletionAudioParamVoiceType1 | str:
            try:
                if not isinstance(data, str):
                    raise TypeError()
                voice_type_1 = ChatCompletionAudioParamVoiceType1(data)

                return voice_type_1
            except (TypeError, ValueError, AttributeError, KeyError):
                pass
            return cast(ChatCompletionAudioParamVoiceType1 | str, data)

        voice = _parse_voice(d.pop("voice"))

        chat_completion_audio_param = cls(
            format_=format_,
            voice=voice,
        )

        chat_completion_audio_param.additional_properties = d
        return chat_completion_audio_param

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from http import HTTPStatus
from typing import Any

import httpx

from ... import errors
from ...client import AuthenticatedClient, Client
from ...models.http_validation_error import HTTPValidationError
from ...models.list_vector_store_files_v1_openai_storage_vector_store_files_list_get_filter_type_0 import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0,
)
from ...models.list_vector_store_files_v1_openai_storage_vector_store_files_list_get_openai_provider import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
)
from ...models.list_vector_store_files_v1_openai_storage_vector_store_files_list_get_order_type_0 import (
    ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0,
)
from ...models.vector_store_file import VectorStoreFile
from ...types import UNSET, Response, Unset


def _get_kwargs(
    *,
    openai_provider: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
    vector_store_id: str,
    limit: int | None | Unset = 10,
    order: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0
    | None
    | Unset = ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
    filter_: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_openai_provider = openai_provider.value
    params["openai_provider"] = json_openai_provider

    params["vector_store_id"] = vector_store_id

    json_limit: int | None | Unset
    if isinstance(limit, Unset):
        json_limit = UNSET
    else:
        json_limit = limit
    params["limit"] = json_limit

    json_order: None | str | Unset
    if isinstance(order, Unset):
        json_order = UNSET
    elif isinstance(order, ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0):
        json_order = order.value
    else:
        json_order = order
    params["order"] = json_order

    json_after: None | str | Unset
    if isinstance(after, Unset):
        json_after = UNSET
    else:
        json_after = after
    params["after"] = json_after

    json_before: None | str | Unset
    if isinstance(before, Unset):
        json_before = UNSET
    else:
        json_before = before
    params["before"] = json_before

    json_filter_: None | str | Unset
    if isinstance(filter_, Unset):
        json_filter_ = UNSET
    elif isinstance(filter_, ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0):
        json_filter_ = filter_.value
    else:
        json_filter_ = filter_
    params["filter"] = json_filter_

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/v1/openai_storage/vector_store_files/list",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> HTTPValidationError | list[VectorStoreFile] | None:
    if response.status_code == 200:
        response_200 = []
        _response_200 = response.json()
        for response_200_item_data in _response_200:
            response_200_item = VectorStoreFile.from_dict(response_200_item_data)

            response_200.append(response_200_item)

        return response_200

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[HTTPValidationError | list[VectorStoreFile]]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
    vector_store_id: str,
    limit: int | None | Unset = 10,
    order: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0
    | None
    | Unset = ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
    filter_: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None | Unset = UNSET,
) -> Response[HTTPValidationError | list[VectorStoreFile]]:
    """List Vector Store Files

    Args:
        openai_provider
            (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider):
        vector_store_id (str):
        limit (int | None | Unset):  Default: 10.
        order (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0 | None |
            Unset):  Default:
            ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC.
        after (None | str | Unset):
        before (None | str | Unset):
        filter_ (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None |
            Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[VectorStoreFile]]
    """

    kwargs = _get_kwargs(
        openai_provider=openai_provider,
        vector_store_id=vector_store_id,
        limit=limit,
        order=order,
        after=after,
        before=before,
        filter_=filter_,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
    vector_store_id: str,
    limit: int | None | Unset = 10,
    order: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0
    | None
    | Unset = ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
    filter_: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None | Unset = UNSET,
) -> HTTPValidationError | list[VectorStoreFile] | None:
    """List Vector Store Files

    Args:
        openai_provider
            (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider):
        vector_store_id (str):
        limit (int | None | Unset):  Default: 10.
        order (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0 | None |
            Unset):  Default:
            ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC.
        after (None | str | Unset):
        before (None | str | Unset):
        filter_ (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None |
            Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[VectorStoreFile]
    """

    return sync_detailed(
        client=client,
        openai_provider=openai_provider,
        vector_store_id=vector_store_id,
        limit=limit,
        order=order,
        after=after,
        before=before,
        filter_=filter_,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
    vector_store_id: str,
    limit: int | None | Unset = 10,
    order: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0
    | None
    | Unset = ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
    filter_: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None | Unset = UNSET,
) -> Response[HTTPValidationError | list[VectorStoreFile]]:
    """List Vector Store Files

    Args:
        openai_provider
            (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider):
        vector_store_id (str):
        limit (int | None | Unset):  Default: 10.
        order (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0 | None |
            Unset):  Default:
            ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC.
        after (None | str | Unset):
        before (None | str | Unset):
        filter_ (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None |
            Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[HTTPValidationError | list[VectorStoreFile]]
    """

    kwargs = _get_kwargs(
        openai_provider=openai_provider,
        vector_store_id=vector_store_id,
        limit=limit,
        order=order,
        after=after,
        before=before,
        filter_=filter_,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    openai_provider: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider,
    vector_store_id: str,
    limit: int | None | Unset = 10,
    order: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0
    | None
    | Unset = ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC,
    after: None | str | Unset = UNSET,
    before: None | str | Unset = UNSET,
    filter_: ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None | Unset = UNSET,
) -> HTTPValidationError | list[VectorStoreFile] | None:
    """List Vector Store Files

    Args:
        openai_provider
            (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOpenaiProvider):
        vector_store_id (str):
        limit (int | None | Unset):  Default: 10.
        order (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0 | None |
            Unset):  Default:
            ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetOrderType0.DESC.
        after (None | str | Unset):
        before (None | str | Unset):
        filter_ (ListVectorStoreFilesV1OpenaiStorageVectorStoreFilesListGetFilterType0 | None |
            Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        HTTPValidationError | list[VectorStoreFile]
    """

    return (
        await asyncio_detailed(
            client=client,
            openai_provider=openai_provider,
            vector_store_id=vector_store_id,
            limit=limit,
            order=order,
            after=after,
            before=before,
            filter_=filter_,
        )
    ).parsed
